@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,12 +32,22 @@ public:
   short Load( File &file );
   int Save( File &file ) const;
 
-  short Check( void );
-  void Execute( class MessageWindow *msgwin );
+  bool Check( void );
+  void Execute( class View *view );
+  void Discard( void );
+  bool Discarded( void ) const { return (e_flags & EFLAG_DISCARDED) != 0; }
 
   unsigned char ID( void ) const { return e_id; }
+  unsigned char Trigger( void ) const { return e_trigger; }
   void SetPlayer( Player &p ) { e_player = &p; }
 
+  short GetData( unsigned short index ) const { return e_data[index]; }
+  void SetData( unsigned short index, short value )
+       { e_data[index] = value; }
+  short GetTData( unsigned short index ) const { return e_tdata[index]; }
+  void SetTData( unsigned short index, short value )
+       { e_tdata[index] = value; }
+
 private:
   bool Disabled( void ) const { return (e_flags & EFLAG_DISABLED) != 0; }
   void SetFlags( unsigned short flags ) { e_flags |= flags; }
@@ -45,15 +55,17 @@ private:
   void ToggleFlags( unsigned short flags ) { e_flags ^= flags; }
 
   void DisplayMessage( Player *p, short msg, short title, class Mission *m,
-                       bool show, class MessageWindow *win ) const;
+                       bool show, class View *view ) const;
 
-  short CheckTrigger( void );
+  bool CheckTrigger( void );
   bool CheckDependencies( TLWList &deps );
+  class Point GetFocus( void ) const;
 
   unsigned char e_id;
   unsigned char e_type;
   unsigned char e_trigger;
   signed char e_depend;
+  signed char e_discard;
 
   short e_tdata[3];             // trigger data
   short e_data[3];              // event data
