@@ -36,14 +36,6 @@
 extern class Game *Gam;
 extern class Image *Images[];
 
-#define ICON_WIDTH	32
-#define ICON_HEIGHT	32
-#define XP_ICON_WIDTH	12
-#define XP_ICON_HEIGHT	14
-
-#define ICON_XP_BASE	0	// XP icons are 0 through 6
-#define NUM_IMAGES	7
-
 #define DEFAULT_DELAY	(5 * ANIM_SPEED_UNIT)
 
 class UndoCache {
