@@ -21,6 +21,7 @@
 ////////////////////////////////////////////////////////////////////////
 
 #include <string.h>
+#include <iostream>
 
 #include "initwindow.h"
 #include "filewindow.h"
@@ -906,7 +907,8 @@ short LocaleOptionsWindow::ReadLocales( void ) {
     string *fname = (string *)n->UserData();
 
     Language *l = new Language();
-    if ( l->ReadCatalog( fname->c_str() ) != -1 ) {
+    short num = l->ReadCatalog( fname->c_str() );
+    if ( num == CF_MSGS ) {
       n->SetName( l->Name() );
       n->SetUserData( l );
       n->SetID( i );
@@ -918,6 +920,14 @@ short LocaleOptionsWindow::ReadLocales( void ) {
 
       ++i;
     } else {
+      if ( num == -1 ) {
+        cerr << "Error: Could not load language resources from "
+             << n->Name() << endl;
+      } else {
+        cerr << "Error: Language catalog '" << n->Name()
+             << "' contains " << num << " strings, expected "
+             << CF_MSGS << endl;
+      }
       delete l;
       delete n;
     }
