@@ -91,7 +91,8 @@ extern Language Lang;
 #define MSG_CHOOSE_PASSWORD	(MSG_ENTER_PASSWORD+1)
 #define MSG_CONFIRM_PASSWORD	(MSG_CHOOSE_PASSWORD+1)
 #define MSG_TRANSFER		(MSG_CONFIRM_PASSWORD+1)
-#define MSG_ASK_SIDE		(MSG_TRANSFER+1)
+#define MSG_TRANSFER_UNITS	(MSG_TRANSFER+1)
+#define MSG_ASK_SIDE		(MSG_TRANSFER_UNITS+1)
 #define MSG_ASK_REPAIR		(MSG_ASK_SIDE+1)
 #define MSG_ASK_OVERWRITE	(MSG_ASK_REPAIR+1)
 #define MSG_ASK_ABORT		(MSG_ASK_OVERWRITE+1)
