@@ -22,6 +22,7 @@
 
 #include "unitwindow.h"
 #include "extwindow.h"
+#include "gamewindow.h"
 #include "game.h"
 #include "strutil.h"
 #include "msgs.h"
@@ -41,23 +42,9 @@
 #define ICON_MINI_TRANSFER2_X	123
 #define ICON_MINI_TRANSFER2_Y	46
 
-
-// scrollable class used for the crystals exchange dialog
-class TransferScrollable : public WidgetHook {
-public:
-  TransferScrollable( short crystals1, short crystals2,
-                      StringWidget *str1, StringWidget *str2,
-                      UnitContainer *c1, UnitContainer *c2 ) 
-  { csum = crystals1 + crystals2; widgets[0] = str1; widgets[1] = str2;
-    container[0] = c1; container[1] = c2; }
-
-  GUI_Status WidgetActivated( Widget *widget, Window *win );
-
-private:
-  short csum;              // sum of crystals in both exchange partners
-  StringWidget *widgets[2];
-  UnitContainer *container[2];
-};
+#define CW_MODE_NORMAL		0
+#define CW_MODE_PRODUCTION	1
+#define CW_MODE_TRANSFER	2
 
 // node class used by the UnitListWidget
 class ULWNode : public Node {
@@ -66,6 +53,7 @@ public:
   Unit *unit;
   unsigned short image;
   bool ok;
+  bool selected;
 };
 
 // list widget used to display units...
@@ -73,53 +61,33 @@ class UnitListWidget : public ListWidget {
 public:
   UnitListWidget( short id, short x, short y, unsigned short w,
     unsigned short h, List *list, short selected,
-    unsigned short flags, MapView *mview, Window *window );
+    unsigned short flags, const UnitSet &uset,
+    const TerrainSet &tset, Window *window );
 
   void DrawNodes( void );
   unsigned short ItemHeight( void ) const { return itemh; }
 
 private:
   unsigned short itemh;
-  MapView *mv;
+  const UnitSet &uset;
+  const TerrainSet &tset;
   Rect hex_bg;
 };
 
 
-// button ID's for the container hook
+// button IDs for the container hook
 #define CH_BUTTON_REPAIR	0
 #define CH_BUTTON_BUILD		1
 #define CH_BUTTON_TRANSFER	2
 
 #define CH_BUTTON_REPAIR_CONFIRM 3
 #define CH_LIST_UNITS		4
+#define CH_SLIDER_CRYSTALS	5
 
-
-////////////////////////////////////////////////////////////////////////
-// NAME       : TransferScrollable::WidgetActivated
-// DESCRIPTION: When a new slider level is set, adjust the TextWidgets
-//              to display the correct amount of crystals to be
-//              transferred.
-// PARAMETERS : widget - activated widget
-//              win    - window the widget is attached to
-// RETURNS    : GUI_OK
-////////////////////////////////////////////////////////////////////////
-
-GUI_Status TransferScrollable::WidgetActivated( Widget *widget, Window *win ) {
-  // current slider level is amount of crystals for the first exchange partner
-  short current = static_cast<SliderWidget *>(widget)->Level();
-  short current2 = csum - current;
-
-  char buf[8];
-
-  widgets[0]->SetString( itoa(current, buf), true );
-  widgets[1]->SetString( itoa(current2, buf), true );
-
-  container[0]->SetCrystals( current );
-  container[1]->SetCrystals( current2 );
-
-  return GUI_OK;
-}
-
+// widget IDs for the UnitLoadWindow
+#define ULW_WIDGET_LIST		1
+#define ULW_WIDGET_OK		2
+#define ULW_WIDGET_CRYSTALS	3
 
 ////////////////////////////////////////////////////////////////////////
 // NAME       : UnitListWidget::UnitListWidget
@@ -132,17 +100,17 @@ GUI_Status TransferScrollable::WidgetActivated( Widget *widget, Window *win ) {
 //              list     - list nodes
 //              selected - node highlighted by default (-1 == none)
 //              flags    - widget flags (see widget.h for details)
-//              mview    - pointer to map viewport
+//              uset     - unit set
+//              tset     - terrain set
 //              window   - widget parent window
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
 UnitListWidget::UnitListWidget( short id, short x, short y, unsigned short w,
     unsigned short h, List *list, short selected, unsigned short flags,
-    MapView *mview, Window *window ) :
+    const UnitSet &uset, const TerrainSet &tset, Window *window ) :
     ListWidget( id, x, y, w, h, list, selected, flags, NULL, window ),
-    hex_bg( 32, 32, ICON_WIDTH, ICON_HEIGHT ) {
-  mv = mview;
+    uset(uset), tset(tset), hex_bg( 32, 32, ICON_WIDTH, ICON_HEIGHT ) {
   spacing = 0;
   itemh = ICON_HEIGHT + spacing;
   Update();
@@ -166,23 +134,27 @@ void UnitListWidget::DrawNodes( void ) {
   surface->DrawBack( area );
 
   while ( n ) {
-    if ( num == current ) {
-      Rect hilite( x + 2, yoff, listboxw - 4, ItemHeight() );
-      hilite.Clip( area );
+    Rect hilite( x + 2, yoff, listboxw - 4, ItemHeight() );
+    hilite.Clip( area );
+
+    if ( n->selected )
+      surface->FillRectAlpha( hilite, Color(CF_COLOR_GHOSTED) );
+
+    if ( num == current )
       surface->FillRectAlpha( hilite, surface->GetFGPen() );
-    }
 
     // draw unit icon
     Rect icon( hex_bg );
     Rect dest( xoff, yoff, ICON_WIDTH, ICON_HEIGHT );
     dest.ClipBlit( icon, area );
+
     icons->Blit( surface, icon, dest.x, dest.y );
-    mv->DrawUnit( n->image, surface, xoff + (ICON_WIDTH - GFX_WIDTH) / 2,
-                  yoff + (ICON_HEIGHT - GFX_HEIGHT) / 2, box );
+    uset.DrawTile( n->image, surface, xoff + (ICON_WIDTH - uset.TileWidth()) / 2,
+                  yoff + (ICON_HEIGHT - uset.TileHeight()) / 2, box );
     if ( !n->ok || (n->unit && !n->unit->IsReady()) )
-      mv->DrawTerrain( IMG_NOT_AVAILABLE, surface,
-                       xoff + (ICON_WIDTH - GFX_WIDTH) / 2,
-                       yoff + (ICON_HEIGHT - GFX_HEIGHT) / 2, box );
+      tset.DrawTile( IMG_NOT_AVAILABLE, surface,
+                     xoff + (ICON_WIDTH - tset.TileWidth()) / 2,
+                     yoff + (ICON_HEIGHT - tset.TileHeight()) / 2, box );
 
     char buf[4];
     if ( n->unit ) {
@@ -217,11 +189,12 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
                  Window( WIN_CLOSE_ESC|WIN_CENTER, view ), c(container),
                  mode(CW_MODE_NORMAL),
                  crystals_icon( view->GetSystemIcons(), 160, 0, ICON_WIDTH, ICON_HEIGHT ),
-                 transferscroll(0), last_selected(0) {
+                 last_selected(0) {
   int i;
   Building *b;
   Transport *t;
   Player *player = dynamic_cast<MapObject *>(c)->Owner();;
+  MapView *mv = Gam->GetMapWindow()->GetMapView();
 
   if ( dynamic_cast<MapObject *>(c)->IsUnit() ) {
     unit = true;
@@ -234,8 +207,8 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
   }
 
   // calculate window dimensions
-  short width = GFX_WIDTH + XP_ICON_WIDTH + lfont->Width() * 20 + 70;
-  short height = 8 * GFX_HEIGHT + 40 + sfont->Height();
+  short width = mv->TileWidth() + XP_ICON_WIDTH + lfont->Width() * 20 + 70;
+  short height = 8 * mv->TileHeight() + 40 + sfont->Height();
   SetSize( MIN(width, view->Width()), MIN(height, view->Height()) );
 
   for ( i = 0; i < CH_NUM_BUTTONS; ++i ) buttons[i] = NULL;
@@ -246,6 +219,7 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
 
     n->unit = c->GetUnit( i );
     n->type = n->unit->Type();
+    n->selected = false;
 
     if ( !player ) {
       n->image = n->unit->Type()->Image() + Gam->GetMission()->GetPlayer().ID() * 6;
@@ -258,11 +232,12 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
   }
 
   // create list widget
-  width = GFX_WIDTH + XP_ICON_WIDTH + DEFAULT_SLIDER_SIZE + 20;   // listwidget width
+  width = mv->TileWidth() + XP_ICON_WIDTH + DEFAULT_SLIDER_SIZE + 20;
   listwidget = new UnitListWidget( CH_LIST_UNITS, 10, 10,
                width, h - sfont->Height() - 28, &normal, -1,
                WIDGET_VSCROLL|WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY,
-               Gam->GetMapWindow()->GetMapView(), this );
+               *mv->GetMap()->GetUnitSet(), *mv->GetMap()->GetTerrainSet(),
+               this );
   listwidget->SetHook( this );
 
   // create buttons
@@ -322,17 +297,6 @@ ContainerWindow::ContainerWindow( UnitContainer *container, View *view ) :
   Show();
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : ContainerWindow::~ContainerWindow
-// DESCRIPTION: Destroy a ContainerWindow instance.
-// PARAMETERS : -
-// RETURNS    : -
-////////////////////////////////////////////////////////////////////////
-
-ContainerWindow::~ContainerWindow( void ) {
-  delete transferscroll;
-}
-
 ////////////////////////////////////////////////////////////////////////
 // NAME       : ContainerWindow::Draw
 // DESCRIPTION: Draw the container window.
@@ -342,7 +306,7 @@ ContainerWindow::~ContainerWindow( void ) {
 
 void ContainerWindow::Draw( void ) {
   Window::Draw();
-  Gam->GetMapWindow()->DrawUnitInfo( NULL, this, unitinfo );
+  UnitInfoWindow::DrawUnitInfo( -1, NULL, NULL, this, unitinfo );
 
   unsigned short xoff;
   Rect boxrect( unitinfo.x, 10, w - unitinfo.x - 10, lfont->Height() + 20 );
@@ -428,6 +392,7 @@ GUI_Status ContainerWindow::HandleEvent( const SDL_Event &event ) {
 ////////////////////////////////////////////////////////////////////////
 
 void ContainerWindow::SwitchMode( unsigned short newmode ) {
+  Mission *m = Gam->GetMission();
 
   if ( newmode == CW_MODE_PRODUCTION ) {             // only buildings, current
                                                      // mode must be CW_MODE_NORMAL
@@ -441,10 +406,11 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
       for ( int i = 0; i < 32; ++i ) {
         if ( prod & (1 << i) ) {        // this unit type can be built
           ULWNode *n = new ULWNode;
-          n->type = Gam->GetMission()->GetUnitSet().GetUnitInfo( i );
+          n->type = m->GetUnitSet().GetUnitInfo( i );
           n->unit = NULL;
           n->image = n->type->Image() + b->Owner()->ID() * 6;
           n->ok = (n->type->Cost() <= crystals);
+          n->selected = false;
           build.AddTail( n );
         }
       }
@@ -456,7 +422,9 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
       // display new list in list widget
       DrawBack( unitinfo );
       listwidget->SwitchList( &build, 0 );
-      Gam->GetMapWindow()->DrawUnitInfo( static_cast<ULWNode *>(build.Head())->type, this, unitinfo );
+      UnitInfoWindow::DrawUnitInfo(
+          static_cast<ULWNode *>(build.Head())->type->ID(),
+          &m->GetUnitSet(), &m->GetTerrainSet(), this, unitinfo );
       Show();
     }
   } else if ( newmode == CW_MODE_TRANSFER ) {       // must be in CW_MODE_NORMAL
@@ -465,46 +433,40 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
     if ( buttons[CH_BUTTON_BUILD] ) buttons[CH_BUTTON_BUILD]->Disable();
     listwidget->Disable();
 
+    // clear unit info area
     DrawBack( unitinfo );
-    Gam->GetMapWindow()->DrawUnitInfo( NULL, this, unitinfo );     // clear unit info area
+    UnitInfoWindow::DrawUnitInfo( -1, NULL, NULL, this, unitinfo );
 
     // create new widgets
     Transport *t = static_cast<Transport *>( static_cast<ULWNode *>(listwidget->Selected())->unit );
     char buf[8];
-    short maxhost, maxclient, fullhost, fullclient, min, max;
-    maxhost = (unit ? c->Slots() * 10 : static_cast<Building *>(c)->CrystalStore());
-    maxclient = t->Slots() * 10;
-    fullhost = c->Crystals();
-    fullclient = t->Crystals();
-    min = MAX( 0, fullhost - (maxclient - fullclient) );
-    max = MIN( maxhost, fullhost + fullclient );
+    unsigned short min, max;
+    min = MAX( 0, c->Crystals() - (t->MaxCrystals() - t->Crystals()) );
+    max = MIN( c->MaxCrystals(), c->Crystals() + t->Crystals() );
 
-    slider = new SliderWidget( 0, unitinfo.x + 10,
+    slider = new SliderWidget( CH_SLIDER_CRYSTALS, unitinfo.x + 10,
                        unitinfo.y + (unitinfo.h - DEFAULT_SLIDER_SIZE) / 2,
                        unitinfo.w - 20, DEFAULT_SLIDER_SIZE,
-                       min, max, fullhost, max - min + 1,
+                       min, max, c->Crystals(), max - min + 1,
                        WIDGET_HSCROLL|WIDGET_HSCROLLKEY, NULL, this );
+    slider->SetHook( this );
     stringwidgets[0] = new StringWidget( 0, slider->LeftEdge(),
                        slider->TopEdge() + slider->Height() + 5,
                        sfont->Width() * 4 + 8, sfont->Height() + 8,
-                       itoa( fullclient, buf ), 4,
+                       itoa( t->Crystals(), buf ), 4,
                        WIDGET_ALIGN_CENTER|WIDGET_STR_CONST, NULL, this );
     stringwidgets[1] = new StringWidget( 0,
                        slider->LeftEdge() + slider->Width() - stringwidgets[0]->Width(),
                        stringwidgets[0]->TopEdge(), stringwidgets[0]->Width(),
-                       stringwidgets[0]->Height(), itoa( fullhost, buf ), 4,
+                       stringwidgets[0]->Height(), itoa( c->Crystals(), buf ), 4,
                        WIDGET_ALIGN_CENTER|WIDGET_STR_CONST, NULL, this );
-    transferscroll = new TransferScrollable( fullhost, fullclient, stringwidgets[1],
-                       stringwidgets[0], c, t );
-    slider->SetHook( transferscroll );
 
     slider->Draw();
     stringwidgets[0]->Draw();
     stringwidgets[1]->Draw();
 
     // draw images
-    Gam->GetMapWindow()->GetMapView()->DrawUnit(
-                   t->BaseImage(), this, slider->LeftEdge(),
+    m->GetUnitSet().DrawTile( t->BaseImage(), this, slider->LeftEdge(),
                    slider->TopEdge() - ICON_HEIGHT - 3, unitinfo );
     crystals_icon.Draw( this, slider->LeftEdge() + slider->Width() - ICON_WIDTH,
                    slider->TopEdge() - ICON_HEIGHT - 3 );
@@ -521,7 +483,7 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
       // display normal list in widget
       DrawBack( unitinfo );
       listwidget->SwitchList( &normal, -1 );
-      Gam->GetMapWindow()->DrawUnitInfo( NULL, this, unitinfo );
+      UnitInfoWindow::DrawUnitInfo( -1, NULL, NULL, this, unitinfo );
       Show();
     } else if ( mode == CW_MODE_TRANSFER ) {          // ...from CW_MODE_TRANSFER
       // delete transfer stuff
@@ -531,13 +493,12 @@ void ContainerWindow::SwitchMode( unsigned short newmode ) {
       delete slider;
       delete stringwidgets[0];
       delete stringwidgets[1];
-      delete transferscroll;
-      transferscroll = NULL;
 
       // redraw unit info
       DrawBack( unitinfo );
-      Gam->GetMapWindow()->DrawUnitInfo( static_cast<ULWNode *>(listwidget->Selected())->type,
-                          this, unitinfo );
+      UnitInfoWindow::DrawUnitInfo(
+          static_cast<ULWNode *>(listwidget->Selected())->type->ID(),
+          &m->GetUnitSet(), &m->GetTerrainSet(), this, unitinfo );
       DrawCrystals();
 
       // re-enable repair and build buttons and list widget
@@ -580,6 +541,7 @@ GUI_Status ContainerWindow::SelectNode( ULWNode *node ) {
         ulw->type = u->Type();
         ulw->image = u->BaseImage();
         ulw->ok = false;
+        ulw->selected = false;
         normal.AddTail( ulw );
 
 	b->SetCrystals( b->Crystals() - u->BuildCost() );
@@ -661,14 +623,28 @@ GUI_Status ContainerWindow::WidgetActivated( Widget *widget, Window *win ) {
                     );
 
     if ( node != last_selected ) {
-      const UnitType *type = NULL;
-      if ( node ) type = node->type;
+      short uid = -1;
+      if ( node ) uid = node->type->ID();
       DrawBack( unitinfo );
-      Gam->GetMapWindow()->DrawUnitInfo( type, this, unitinfo );
+      UnitInfoWindow::DrawUnitInfo( uid, &Gam->GetMission()->GetUnitSet(),
+                      &Gam->GetMission()->GetTerrainSet(), this, unitinfo );
       win->Show( unitinfo );
       last_selected = node;
     } else if ( node ) rc = SelectNode( node );
     break; }
+  case CH_SLIDER_CRYSTALS: {
+    // current slider level is amount of crystals for the first exchange partner
+    Transport *t = static_cast<Transport *>( static_cast<ULWNode *>(listwidget->Selected())->unit );
+    short current = slider->Level();
+    short current2 = t->Crystals() + c->Crystals() - current;
+
+    char buf[8];
+    stringwidgets[1]->SetString( itoa(current, buf), true );
+    stringwidgets[0]->SetString( itoa(current2, buf), true );
+
+    c->SetCrystals( current );
+    t->SetCrystals( current2 );
+    break; }
   }
 
   return rc;
@@ -676,86 +652,204 @@ GUI_Status ContainerWindow::WidgetActivated( Widget *widget, Window *win ) {
 
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : UnitInfoWindow::UnitInfoWindow
-// DESCRIPTION: Pop up a window with information about the given unit
-//              type.
-// PARAMETERS : type   - unit type
-//              mapwin - pointer to map window
-//              view   - view the window will be attached to
+// NAME       : UnitLoadWindow::UnitLoadWindow
+// DESCRIPTION: This window is used to ask the player which units to
+//              load when he moves a transport out of a container.
+// PARAMETERS : t    - transport to be moved
+//              c    - unit container the transport is moving out of
+//              uset - unit set used
+//              tset - terrain set used
+//              view - pointer to the view
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-UnitInfoWindow::UnitInfoWindow( const UnitType *type, MapWindow *mapwin,
-        View *view ) : Window(WIN_CENTER, view), mw(mapwin), unit(type) {
-  unsigned short width = MAX( sfont->Width() * 24, GFX_WIDTH * 4) + 20,
-                 height = GFX_HEIGHT + GFX_OVERLAP_Y +
-                          ICON_HEIGHT * 2 + sfont->Height() + 30;
+UnitLoadWindow::UnitLoadWindow( Transport &t, UnitContainer &c,
+        const UnitSet &uset, const TerrainSet &tset, View *view ) :
+        Window( WIN_CENTER, view ), t(t), c(c), last_selected(0),
+        fullslots(t.FullSlots()), crystals(t.Crystals()), have_units(false) {
+  // calculate window dimensions
+  short width = uset.TileWidth() + XP_ICON_WIDTH + DEFAULT_SLIDER_SIZE +
+                MAX(sfont->Width() * 24, tset.TileWidth() * 4) + 20;
+  short height = tset.TileHeight() + tset.TileShiftY() + ICON_HEIGHT * 3 +
+                 sfont->Height() * 3 + 70;
+  SetSize( MIN(width, view->Width()), MIN(height, view->Height()) );
 
-  // don't show portrait on small screens
-  short ptnum = type->Portrait();
-  if ( (view->Width() < 640) || (view->Height() < 480) ) ptnum = -1;
-
-  if ( ptnum < 0 ) width = MAX( width, GFX_WIDTH * 7 + 40 );
-  info.x = info.y = 5;
-  info.w = width - 10;
-  info.h = height - 10;
-
-
-  if ( ptnum >= 0 ) {
-    portrait = mw->GetMapView()->GetMap()->GetUnitSet()->GetPortrait(ptnum);
-    image.w = portrait->Width() + 2;
-    image.h = portrait->Height() + 2;
-
-    if ( image.w >= image.h ) {     // put image below info area
-      width = MAX( width, image.w + 10 );
-      height += image.h + 5;
-      image.x = (width - image.w) / 2;
-      image.y = height - 5 - image.h;
-      info.w = width - 10;
-    } else {                        // draw image right of info area
-      width += image.w + 5;
-      height = MAX( height, image.h + 10 );
-      image.x = width - image.w - 5;
-      image.y = (height - image.h) / 2;
-      info.h = height - 10;
+  // create unit list
+  for ( int i = 0; i < c.UnitCount(); ++i ) {
+    Unit *u = c.GetUnit( i );
+    if ( (u != &t) && static_cast<UnitContainer &>(t).Allow( u->Type() ) ) {
+      ULWNode *n = new ULWNode;
+      n->unit = u;
+      n->type = u->Type();
+      n->image = u->BaseImage();
+      n->ok = u->IsReady() && t.Allow( u );
+      n->selected = false;
+      units.AddTail( n );
+      have_units |= n->ok;
     }
-  } else portrait = NULL;
+  }
 
-  SetSize( width, height );
+  if ( !have_units ) return;
+
+  // create list widget
+  wd_list = new UnitListWidget(
+          ULW_WIDGET_LIST, 10, sfont->Height() + 10,
+          uset.TileWidth() + XP_ICON_WIDTH + DEFAULT_SLIDER_SIZE + 20,
+          h - sfont->Height() * 2 - 28, &units, -1,
+          WIDGET_VSCROLL|WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY,
+          uset, tset, this );
+  wd_list->SetKey( SDLK_SPACE );
+  wd_list->SetHook( this );
+
+  // create buttons
+  Widget *wd = new ButtonWidget( ULW_WIDGET_OK,
+               1, h - sfont->Height() - 9, (w - 2) / 2, sfont->Height() + 8,
+               WIDGET_DEFAULT, MSG(MSG_B_OK), this );
+  wd->SetHook( this );
+
+  new ButtonWidget( GUI_CLOSE, wd->LeftEdge() + wd->Width(), wd->TopEdge(),
+               wd->Width(), wd->Height(), 0, MSG(MSG_B_CANCEL), this );
+
+  unitinfo = Rect( wd_list->LeftEdge() + wd_list->Width() + 5,
+             wd_list->TopEdge(),
+             w - wd_list->LeftEdge() - wd_list->Width() - 15,
+             wd_list->Height() - ICON_HEIGHT );
+
+  char buf[10];
+  sprintf( buf, "(%d)", c.Crystals() + t.Crystals() );
+  cstrw = sfont->TextWidth( buf );
+  wd_crystals = new StringWidget( 0,
+           unitinfo.x + unitinfo.w - cstrw - sfont->Width() * 4,
+           unitinfo.y + unitinfo.h + (ICON_HEIGHT - sfont->Height() - 8)/2,
+           sfont->Width() * 4, sfont->Height() + 8,
+           itoa( crystals, buf ), 4,
+           WIDGET_ALIGN_CENTER|WIDGET_STR_CONST|WIDGET_STYLE_NOBORDER,
+           NULL, this );
+
+  unsigned short max = MIN( t.MaxCrystals() - ((fullslots - (crystals+9)/10) * 10),
+                            c.Crystals() + t.Crystals() );
+  wd_slider = new SliderWidget( ULW_WIDGET_CRYSTALS, unitinfo.x + ICON_WIDTH,
+           wd_crystals->TopEdge() + (wd_crystals->Height() - DEFAULT_SLIDER_SIZE) / 2,
+           wd_crystals->LeftEdge() - unitinfo.x - ICON_WIDTH - 5, DEFAULT_SLIDER_SIZE,
+           0, max, crystals, unitinfo.w / (max+1),
+           WIDGET_HSCROLL|WIDGET_HSCROLLKEY, NULL, this );
+  wd_slider->SetHook( this );
 
   Draw();
+  wd_list->Select( 0 );
   Show();
 }
 
 ////////////////////////////////////////////////////////////////////////
-// NAME       : UnitInfoWindow::Draw
+// NAME       : UnitLoadWindow::Draw
 // DESCRIPTION: Draw the window.
 // PARAMETERS : -
 // RETURNS    : -
 ////////////////////////////////////////////////////////////////////////
 
-void UnitInfoWindow::Draw( void ) {
+void UnitLoadWindow::Draw( void ) {
   Window::Draw();
-  mw->DrawUnitInfo( unit, this, info );
-  if ( portrait ) {
-    DrawBox( image, BOX_RECESSED );
-    portrait->Blit( this, *portrait, image.x + 1, image.y + 1 );
+  UnitInfoWindow::DrawUnitInfo( -1, NULL, NULL, this, unitinfo );
+
+  const char *msg = MSG(MSG_TRANSFER_UNITS);
+  unsigned short txtw = sfont->TextWidth(msg);
+  sfont->WriteEllipsis( msg, this, MAX( 10, (w - txtw) / 2 ), 5,
+                        Rect( 10, 5, w-20, sfont->Height() ) );
+
+  Image crystals_icon( view->GetSystemIcons(), 160, 0, ICON_WIDTH, ICON_HEIGHT );
+  crystals_icon.Draw( this, unitinfo.x, unitinfo.y + unitinfo.h );
+
+  char buf[10];
+  sprintf( buf, "(%d)", c.Crystals() + t.Crystals() );
+  sfont->Write( buf, this, unitinfo.x + unitinfo.w - cstrw,
+                unitinfo.y + unitinfo.h + (ICON_HEIGHT - sfont->Height() - 2)/2,
+                *this );
+}
+
+////////////////////////////////////////////////////////////////////////
+// NAME       : UnitLoadWindow::WidgetActivated
+// DESCRIPTION: Call the appropriate functions when widgets are pressed
+//              in the window.
+// PARAMETERS : widget - widget that called the function
+//              win    - pointer to the window the widget belongs to
+// RETURNS    : GUI status
+////////////////////////////////////////////////////////////////////////
+
+GUI_Status UnitLoadWindow::WidgetActivated( Widget *widget, Window *win ) {
+  GUI_Status rc = GUI_OK;
+  ULWNode *node;
+
+  switch ( widget->ID() ) {
+  case ULW_WIDGET_LIST: {
+    // Display information about the currently selected unit
+    // or, if the node has been selected twice (e.g. by double
+    // clicking), select the respective unit.
+    node = static_cast<ULWNode *>( wd_list->Selected() );
+
+    if ( node != last_selected ) {
+      short uid = -1;
+      if ( node ) uid = node->type->ID();
+      DrawBack( unitinfo );
+      UnitInfoWindow::DrawUnitInfo( uid, &Gam->GetMission()->GetUnitSet(),
+                      &Gam->GetMission()->GetTerrainSet(), this, unitinfo );
+      win->Show( unitinfo );
+      last_selected = node;
+    } else if ( node ) {
+      if ( node->ok ) {
+        node->selected ^= 1;
+        if ( node->selected ) fullslots += node->unit->Weight();
+        else fullslots -= node->unit->Weight();
+
+        // update the ok flag for all nodes
+        for ( node = static_cast<ULWNode *>(units.Head()); node;
+              node = static_cast<ULWNode *>(node->Next()) ) {
+          node->ok = node->selected || (node->unit->IsReady() &&
+                     (t.Slots() >= fullslots + node->unit->Weight()));
+        }
+
+        wd_list->Draw();
+        wd_list->Show();
+
+        unsigned short max =
+          MIN( t.MaxCrystals() - ((fullslots - (crystals+9)/10) * 10),
+               c.Crystals() + t.Crystals() );
+        wd_slider->Adjust( 0, max, unitinfo.w / (max+1) );
+        wd_slider->Draw();
+        wd_slider->Show();
+      } else Audio::PlaySfx( Audio::SND_GUI_ERROR, 0 );
+    }
+    break; }
+  case ULW_WIDGET_CRYSTALS: {
+    char buf[8];
+    fullslots -= (crystals + 9)/10 - (wd_slider->Level() + 9)/10;
+    crystals = wd_slider->Level();
+    wd_crystals->SetString( itoa( crystals, buf ), true );
+
+    // update the ok flag for all nodes
+    for ( node = static_cast<ULWNode *>(units.Head()); node;
+          node = static_cast<ULWNode *>(node->Next()) ) {
+      node->ok = node->selected || (node->unit->IsReady() &&
+                 (t.Slots() >= fullslots + node->unit->Weight()));
+    }
+    wd_list->Draw();
+    wd_list->Show();
+    break; }
+  case ULW_WIDGET_OK:
+    // move selected units into the transport
+    for ( node = static_cast<ULWNode *>(units.Head()); node;
+          node = static_cast<ULWNode *>(node->Next()) ) {
+      if ( node->selected ) {
+        c.RemoveUnit( node->unit );
+        node->unit->SetFlags( U_MOVED );
+        t.InsertUnit( node->unit );
+      }
+    }
+    c.SetCrystals( c.Crystals() - (crystals - t.Crystals()) );
+    t.SetCrystals( crystals );
+    rc = (GUI_Status)ULW_WIDGET_OK;
+    break;
   }
-}
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : UnitInfoWindow::HandleEvent
-// DESCRIPTION: Close the window on any button or key pressed.
-// PARAMETERS : event - event received by the event handler
-// RETURNS    : GUI_CLOSE if the window should be closed,
-//              GUI_OK otherwise
-////////////////////////////////////////////////////////////////////////
-
-GUI_Status UnitInfoWindow::HandleEvent( const SDL_Event &event ) {
-
-  if ( (event.type == SDL_KEYDOWN) || (event.type == SDL_MOUSEBUTTONDOWN) )
-    return GUI_CLOSE;
-
-  return GUI_OK;
+  return rc;
 }
 
