@@ -28,29 +28,19 @@
 #include "button.h"
 #include "listselect.h"
 #include "textbox.h"
-#include "container.h"
-#include "building.h"
+#include "map.h"
 
-class Game;
-class MapWindow;
-class UnitSet;
-class TerrainSet;
 class UnitListWidget;
-class TransferScrollable;
 class ULWNode;
 
 // the same window class is used for buildings and transports
 
-#define CW_MODE_NORMAL		0
-#define CW_MODE_PRODUCTION	1
-#define CW_MODE_TRANSFER	2
-
 #define CH_NUM_BUTTONS		4
 
 class ContainerWindow : public Window, public WidgetHook {
 public:
   ContainerWindow( UnitContainer *container, View *view );
-  ~ContainerWindow( void );
+  ~ContainerWindow( void ) {}
 
   GUI_Status HandleEvent( const SDL_Event &event );
   void SwitchMode( unsigned short newmode );
@@ -78,27 +68,35 @@ private:
   ButtonWidget *buttons[CH_NUM_BUTTONS];
   SliderWidget *slider;
   StringWidget *stringwidgets[2];
-  TransferScrollable *transferscroll;
 
   Node *last_selected;
 };
 
-
-// unit type information window
-class UnitInfoWindow : public Window {
+class UnitLoadWindow : public Window, public WidgetHook {
 public:
-  UnitInfoWindow( const UnitType *type, MapWindow *mapwin,
+  UnitLoadWindow( Transport &t, UnitContainer &c,
+                  const UnitSet &uset, const TerrainSet &tset,
                   View *view );
 
   void Draw( void );
-  GUI_Status HandleEvent( const SDL_Event &event );
+  GUI_Status WidgetActivated( Widget *widget, Window *win );
+
+  bool Opened( void ) const { return have_units; }
 
 private:
-  Rect image;
-  Rect info;
-  MapWindow *mw;
-  const UnitType *unit;
-  Surface *portrait;
+  Transport &t;
+  UnitContainer &c;
+  List units;
+  ULWNode *last_selected;
+  unsigned short fullslots; // we can't change these in the transport
+  unsigned short crystals;  // because the user may cancel
+  bool have_units;
+
+  StringWidget *wd_crystals;
+  SliderWidget *wd_slider;
+  ListWidget *wd_list;
+  Rect unitinfo;
+  unsigned short cstrw;
 };
 
 #endif	/* _INCLUDE_UNITWINDOW_H */
