@@ -29,6 +29,7 @@ extern const char *player2_labels[];
 extern const char *dir_labels[];
 extern const char *size_labels[];
 extern const char *xp_labels[];
+static const char *uselect_labels[] = { "Single Unit", "Unit Type", 0 };
 
 // initialize static class variables
 const short EdEventGenericWindow::B_ID_TITLE = 1;
@@ -102,6 +103,7 @@ const short EdTrigHaveUnitWindow::N_ID_TIMER = 4;
 const short EdTrigUnitDestroyedWindow::B_ID_UNIT = 1;
 const short EdTrigUnitDestroyedWindow::B_ID_UNIT_OK = 2;
 const short EdTrigUnitDestroyedWindow::B_ID_PLAYER = 3;
+const short EdTrigUnitDestroyedWindow::B_ID_USELECT = 4;
 
 const short EdTrigUnitPositionWindow::B_ID_UNIT = 1;
 const short EdTrigUnitPositionWindow::B_ID_UNIT_OK = 2;
@@ -1026,10 +1028,13 @@ GUI_Status EdEventScoreWindow::WidgetActivated( Widget *widget, Window *win ) {
 EdEventSetHexWindow::EdEventSetHexWindow( Event &event, Mission &mission,
                                           View *view ) :
       Window(WIN_CENTER, view), event(event), mission(mission) { 
-  SetSize( MIN(sfont->Width() * 15 + 15, view->Width()),
-           sfont->Height() * 2 + GFX_HEIGHT + 30 );
+  const TerrainSet &set = mission.GetTerrainSet();
 
-  tilepos = Rect( (w - GFX_WIDTH) / 2, 5, GFX_WIDTH, GFX_HEIGHT );
+  SetSize( MIN(sfont->Width() * 15 + 15, view->Width()),
+           sfont->Height() * 2 + set.TileHeight() + 30 );
+
+  tilepos = Rect( (w - set.TileWidth()) / 2, 5,
+                  set.TileWidth(), set.TileHeight() );
 
   Widget *wd;
   unsigned short wdx, wdh = sfont->Height() + 8;
@@ -1090,14 +1095,14 @@ GUI_Status EdEventSetHexWindow::WidgetActivated( Widget *widget, Window *win ) {
   case B_ID_TILE_PREV:
     num = event.GetData( 0 );
     if ( num > 0 ) --num;
-    else num = mission.GetTerrainSet().NumTT() - 1;
+    else num = mission.GetTerrainSet().NumTiles() - 1;
     event.SetData( 0, num );
     DrawTerrain( num, true );
     Show( tilepos );
     break;
   case B_ID_TILE_NEXT:
     num = event.GetData( 0 );
-    if ( ++num == mission.GetTerrainSet().NumTT() ) num = 0;
+    if ( ++num == mission.GetTerrainSet().NumTiles() ) num = 0;
     event.SetData( 0, num );
     DrawTerrain( num, true );
     Show( tilepos );
@@ -1125,12 +1130,11 @@ GUI_Status EdEventSetHexWindow::WidgetActivated( Widget *widget, Window *win ) {
 ////////////////////////////////////////////////////////////////////////
 
 void EdEventSetHexWindow::DrawTerrain( unsigned short terrain, bool update ) {
-  TerrainSet &set = mission.GetTerrainSet();
+  const TerrainSet &set = mission.GetTerrainSet();
   const TerrainType *tt = set.GetTerrainInfo( terrain );
 
   if ( tt ) {
-    MapView::DrawHex( tt->tt_image, set.GetTiles(), this,
-                           tilepos.x, tilepos.y, tilepos );
+    set.DrawTile( tt->tt_image, this, tilepos.x, tilepos.y, tilepos );
 
     if ( update ) Show( tilepos );
   }
@@ -1444,26 +1448,36 @@ EdTrigUnitDestroyedWindow::EdTrigUnitDestroyedWindow( Event &event,
                            Mission &mission, View *view ) :
       Window(WIN_CENTER, view), event(event), mission(mission) { 
   SetSize( MIN(sfont->Width() * 30 + 15, view->Width()),
-           sfont->Height() * 3 + 40 );
+           sfont->Height() * 4 + 50 );
 
   Widget *wd;
   unsigned short wdx, wdh = sfont->Height() + 8;
   char buf[32] = "All";
 
-  wd = new ButtonWidget( B_ID_UNIT, 5, 5,
+  wdx = 10 + sfont->TextWidth("Select");
+  t_uselect = new CycleWidget( B_ID_USELECT, wdx, 5, w/2 - wdx - 5, wdh,
+              0, "_Select", (event.GetTData(0) >= -1) ? 0 : 1,
+              uselect_labels, this );
+  t_uselect->SetHook( this );
+
+  wd = new ButtonWidget( B_ID_UNIT,
+       5, t_uselect->TopEdge() + wdh + 5,
        sfont->Width() * 4 + 10, wdh, 0, "_Unit", this );
   wd->SetHook( this );
 
   if ( event.GetTData(0) >= 0 ) {
     Unit *u = mission.GetUnitByID( event.GetTData(0) );
     if ( u ) sprintf( buf, "%s (%d)", u->Name(), u->ID() );
+  } else if ( event.GetTData(0) < -1 ) {
+    strcpy( buf, mission.GetUnitSet().GetUnitInfo(-event.GetTData(0) - 2)->Name() );
   }
   wdx = wd->LeftEdge() + wd->Width() + 5;
-  t_unit = new StringWidget( 0, wdx, 5, w - wdx - 5, wdh,
+  t_unit = new StringWidget( 0, wdx, wd->TopEdge(), w - wdx - 5, wdh,
            buf, 30, WIDGET_STR_CONST, NULL, this );
 
   wdx = sfont->Width() * 20 + 10;
-  t_player = new CycleWidget( B_ID_PLAYER, wdx, wd->Height() + 10,
+  t_player = new CycleWidget( B_ID_PLAYER,
+             wdx, wd->TopEdge() + wd->Height() + 5,
              w - wdx - 5, wdh,
              (event.GetTData(0) >= 0) ? WIDGET_DISABLED : 0,
              "Controlled by _Player", event.GetTData(1),
@@ -1488,28 +1502,54 @@ GUI_Status EdTrigUnitDestroyedWindow::WidgetActivated( Widget *widget, Window *w
 
   switch( widget->ID() ) {
   case B_ID_UNIT:
-    TLWListBuilder::BuildUnitList( mission.GetUnits(), tlist );
-    new ListSelectWindow( "Select target unit", tlist,
-        event.GetTData(0), true, this, B_ID_UNIT_OK, view );
+    if ( t_uselect->GetValue() == 0 ) {
+      TLWListBuilder::BuildUnitList( mission.GetUnits(), tlist );
+      new ListSelectWindow( "Select target unit", tlist,
+          event.GetTData(0), true, this, B_ID_UNIT_OK, view );
+    } else {
+      TLWListBuilder::BuildUnitTypesList( mission.GetUnitSet(), tlist );
+      new ListSelectWindow( "Select unit type", tlist,
+          -event.GetTData(0) - 2, true, this, B_ID_UNIT_OK, view );
+    }
     break;
   case B_ID_UNIT_OK: {
     TLWNode *n = static_cast<ListSelectWindow *>(win)->Selected();
-    event.SetTData( 0, n ? n->ID() : -1 );
-    t_unit->SetString( n ? n->Name() : "All", true );
-
-    if ( n ) {
-      event.SetTData( 1, static_cast<Unit *>(n->UserData())->Owner() );
-      t_player->Disable();
+    if ( t_uselect->GetValue() == 0 ) {
+      event.SetTData( 0, n ? n->ID() : -1 );
+      if ( n ) {
+        event.SetTData( 1, static_cast<Unit *>(n->UserData())->Owner() );
+        t_player->Disable();
+      } else {
+        event.SetTData( 1, event.Player()^1 );
+        t_player->Enable();
+        t_player->SetValue( event.GetTData(1) );
+      }
     } else {
-      event.SetTData( 1, event.Player()^1 );
-      t_player->Enable();
-      t_player->SetValue( event.GetTData(1) );
+      event.SetTData( 0, n ? -n->ID() - 2 : -1);
     }
+    t_unit->SetString( n ? n->Name() : "All", true );
 
     Draw();
     Show();
     tlist.Clear();
     break; }
+  case B_ID_USELECT:
+    if ( t_uselect->GetValue() == 0 ) {
+      event.SetTData( 0, -1 );
+      t_unit->SetString( "All", true );
+    } else if ( event.GetTData(0) != -1 ) {
+      Unit *u = mission.GetUnitByID( event.GetTData(0) );
+      if ( u ) {
+        event.SetTData( 0, -u->Type()->ID() - 2 );
+        t_unit->SetString( u->Name(), true );
+      } else {
+        event.SetTData( 0, -1 );
+        t_unit->SetString( "All", true );
+      }
+    }
+    t_player->Enable();
+    t_player->SetValue( event.GetTData(1) );
+    break;
   case B_ID_PLAYER:
     event.SetTData( 1, static_cast<CycleWidget *>(widget)->GetValue() );
   }
@@ -1536,7 +1576,6 @@ EdTrigUnitPositionWindow::EdTrigUnitPositionWindow( Event &event,
   Widget *wd;
   unsigned short wdx, wdh = sfont->Height() + 8;
   char buf[32] = "Any";
-  static const char *uselect_labels[] = { "Single Unit", "Unit Type", 0 };
   Point loc = mission.GetMap().Index2Hex( event.GetTData(1) );
 
   wdx = 10 + sfont->TextWidth("Select");
