@@ -786,13 +786,13 @@ EdBuildingWindow::EdBuildingWindow( Building &bld, Mission &mission, View *view
 
   xoff = 10 + sfont->TextWidth("Crystals");
   b_crystals = new NumberWidget( 0, xoff, yoff, sfont->Width() * 4 + 10, wdh,
-                                 b.Crystals(), 0, b.CrystalStore(),
+                                 b.Crystals(), 0, b.MaxCrystals(),
                                  WIDGET_ALIGN_LEFT, "_Crystals", this );
 
   xoff += b_crystals->Width() + 10 + sfont->TextWidth("of");
   b_maxcrystals = new NumberWidget( N_ID_MAXCRYSTALS, xoff, yoff,
                                   b_crystals->Width(), wdh,
-                                  b.CrystalStore(), 0, 10000,
+                                  b.MaxCrystals(), 0, 10000,
                                   WIDGET_ALIGN_LEFT, "of", this );
   b_maxcrystals->SetHook( this );
 
