@@ -1,5 +1,5 @@
 // CoMET - The Crimson Fields Map Editing Tool
-// Copyright (C) 2002-2004 Jens Granseuer
+// Copyright (C) 2002-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -271,3 +271,22 @@ const TerrainType *Map::HexType( const Point &hex ) const {
   return m_tset->GetTerrainInfo(HexTypeID(hex));
 }
 
+////////////////////////////////////////////////////////////////////////
+// NAME       : Map::Index2Hex
+// DESCRIPTION: Turn a hex index back into a Point.
+// PARAMETERS : index - hex index
+// RETURNS    : hex coordinates (-1/-1 on invalid index)
+////////////////////////////////////////////////////////////////////////
+
+Point Map::Index2Hex( int index ) const {
+  Point p(-1, -1);
+
+  if ( index >= 0 ) {
+    p.x = index % Width();
+    p.y = index / Width();
+
+    if ( (p.x >= Width()) || (p.y >= Height()) ) return Point(-1, -1);
+  }
+  return p;
+}
+
