@@ -869,7 +869,7 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
     break;
 
   case ETRIGGER_UNIT_DESTROYED:
-    if ( e.GetTData(0) != -1 ) {
+    if ( e.GetTData(0) >= 0 ) {
       Unit *u = GetUnitByID( e.GetTData(0) );
 
       if ( !u ) {
@@ -891,7 +891,7 @@ unsigned short Mission::ValidateEvent( Event &e, stringstream &errors ) const {
       ++errs;
       errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
              << ") event targets non-existing unit with ID " << e.GetTData(0) << ".\n\n";
-    } else if ( (e.GetTData(0) < -1) && (-e.GetTData(0) - 2 >= unit_set->NumUT()) ) {
+    } else if ( (e.GetTData(0) < -1) && (-e.GetTData(0) - 2 >= unit_set->NumTiles()) ) {
       ++errs;
       errors << "Error: Trigger of " << e.Name() << " (" << (short)e.ID()
              << ") event targets invalid unit type with ID " << -e.GetTData(0) - 2 << ".\n\n";
@@ -1266,7 +1266,9 @@ int Event::Export( ofstream &file, const Map &map ) const {
     break;
   case ETRIGGER_UNIT_DESTROYED:
     file << "unitdestroyed\n";
-    file << "tunit = " << e_tdata[0] << '\n';
+    file << "tunit = ";
+    if ( e_tdata[0] >= -1 ) file << e_tdata[0] << '\n';
+    else file << uset->GetUnitInfo(-e_tdata[0] - 2)->Name() << '\n';
     file << "towner = " << e_tdata[1] + 1 << '\n';
     break;
   case ETRIGGER_HAVE_BUILDING:
