@@ -117,6 +117,14 @@
 #define MCOST_MIN		1  /* cost to cross a road hex for ground units
 				      or any hex type for airborne units */
 
+#define ICON_WIDTH	32
+#define ICON_HEIGHT	32
+#define XP_ICON_WIDTH	12
+#define XP_ICON_HEIGHT	14
+
+#define ICON_XP_BASE	0	// XP icons are 0 through 6
+#define NUM_IMAGES	7
+
 /* building definitions */
 #define BLD_WORKSHOP	0x0001    /* units can be repaired */
 #define BLD_FACTORY	0x0002    /* units can be produced */
