@@ -242,7 +242,9 @@ GUI_Status ListWidget::KeyDown( const SDL_keysym &key ) {
 
   if ( rc != GUI_OK ) return rc;
 
-  if ( flags & (WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY) ) {
+  if ( key.sym == this->key ) rc = Select( current );
+
+  else if ( flags & (WIDGET_HSCROLLKEY|WIDGET_VSCROLLKEY) ) {
     switch ( key.sym ) {
     case SDLK_UP:
       if ( (current > 0) && (flags & WIDGET_VSCROLLKEY) )
@@ -263,6 +265,6 @@ GUI_Status ListWidget::KeyDown( const SDL_keysym &key ) {
     }
   }
 
-  return GUI_OK;
+  return rc;
 }
 
