@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,12 +23,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#ifdef HAVE_LIBZ
-# include <zlib.h>
-#else
-# include <stdio.h>
-#endif
-
 #include "misc.h"
 
 ////////////////////////////////////////////////////////////////////////
@@ -97,24 +91,3 @@ char *itoa( int n, char *buf ) {
   return buf;
 }
 
-////////////////////////////////////////////////////////////////////////
-// NAME       : unpack
-// DESCRIPTION: Uncompress some data. The function uses zlib internally.
-// PARAMETERS : dest    - destination buffer; must be large enough to
-//                        hold the uncompressed data
-//              destlen - length of dest
-//              src     - buffer containing the compressed data
-//              srclen  - length of the src
-// RETURNS    : 0 on success, -1 on error
-////////////////////////////////////////////////////////////////////////
-
-int unpack( unsigned char *dest, unsigned long destlen,
-            const unsigned char *src, unsigned long srclen ) {
-#ifdef HAVE_LIBZ
-  return uncompress( dest, &destlen, src, srclen );
-#else
-  fprintf( stderr, "Error: Trying to use compressed data files without zlib support\n" );
-  return 0;
-#endif
-}
-
