@@ -1,5 +1,5 @@
 // Crimson Fields -- a game of tactical warfare
-// Copyright (C) 2000-2004 Jens Granseuer
+// Copyright (C) 2000-2005 Jens Granseuer
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -145,23 +145,24 @@ GUI_Status SliderWidget::MouseDown( const SDL_MouseButtonEvent &button ) {
   short my = button.y - surface->TopEdge();
 
   if ( Contains( mx, my ) ) {
+    short step = MAX( 1, size - 1 );
     if ( button.button == SDL_BUTTON_LEFT ) {
       if ( knob.Contains( mx, my ) ) {
         Push();
         mousehit = true;
       } else {
         if ( flags & WIDGET_HSCROLL ) {
-          if ( mx < knob.x ) ScrollTo( current - size + 1 );
-          else if ( mx >= (knob.x + knob.w) ) ScrollTo( current + size - 1 );
+          if ( mx < knob.x ) ScrollTo( current - step );
+          else if ( mx >= (knob.x + knob.w) ) ScrollTo( current + step );
         } else {
-          if ( my < knob.y ) ScrollTo( current - size + 1 );
-          else if ( my >= (knob.y + knob.h) ) ScrollTo( current + size - 1 );
+          if ( my < knob.y ) ScrollTo( current - step );
+          else if ( my >= (knob.y + knob.h) ) ScrollTo( current + step );
         }
       }
     } else if ( button.button == SDL_BUTTON_WHEELUP ) {
-      ScrollTo( current - size + 1 );
+      ScrollTo( current - step );
     } else if ( button.button == SDL_BUTTON_WHEELDOWN ) {
-      ScrollTo( current + size - 1 );
+      ScrollTo( current + step );
     }
   }
 
