@@ -53,6 +53,7 @@ public:
                 { return SDL_SetAlpha( s_surface, flags, alpha ); }
   int SetColorKey( const Color &col );
   Color GetColorKey( void ) const;
+  Color GetPixel( unsigned short x, unsigned short y ) const;
   unsigned long MapRGB( const Color &col ) const
                 { return SDL_MapRGB( s_surface->format, col.r, col.g, col.b ); }
 
@@ -94,6 +95,19 @@ protected:
   void DrawPixel( short const x, short const y, const Color &col ) const;
 };
 
+class SurfaceLock {
+public:
+  SurfaceLock( const Surface *surface ) : s(surface->s_surface) {
+    locked = ( SDL_MUSTLOCK( s ) && (SDL_LockSurface( s ) == 0) );
+  }
+
+  ~SurfaceLock( void ) { if ( locked ) SDL_UnlockSurface( s ); }
+
+private:
+  SDL_Surface *s;
+  bool locked;
+};
+
 
 class Image : public Rect {
 public:
