@@ -151,7 +151,7 @@ const UnitType *CfedParserUtils::ParseUnitName( const string &name ) const {
   const UnitType *ut = NULL;
   const UnitSet &us = m.GetUnitSet();
 
-  for ( int i = 0; i < us.NumUT(); ++i ) {
+  for ( int i = 0; i < us.NumTiles(); ++i ) {
     if ( name == us.UnitName(i) ) {
       ut = us.GetUnitInfo(i);
       break;
@@ -494,8 +494,7 @@ int EventHandler::ParseSection( ifstream &in, const string *opt, unsigned long &
       else {
         if ( (e->Trigger() == ETRIGGER_TIMER) &&
              (key == "ttime") ) e->SetTData(0, StrToNum(val));      // timer
-        else if ( ((e->Trigger() == ETRIGGER_UNIT_DESTROYED) ||
-                   (e->Trigger() == ETRIGGER_HAVE_UNIT)) &&
+        else if ( (e->Trigger() == ETRIGGER_HAVE_UNIT) &&
              (key == "tunit") ) e->SetTData(0, StrToNum(val));      // unit
         else if ( (e->Trigger() == ETRIGGER_HAVE_BUILDING) &&
              (key == "tbuilding") ) e->SetTData(0, StrToNum(val));  // building id
@@ -515,7 +514,8 @@ int EventHandler::ParseSection( ifstream &in, const string *opt, unsigned long &
             cerr << "Error near line " << line << ": Invalid position " << val << endl;
           } else e->SetTData(1, m.GetMap().Hex2Index(loc));         // unit position
         }
-        else if ( (e->Trigger() == ETRIGGER_UNIT_POSITION) && (key == "tunit") ) { // unit or unit type
+        else if ( ((e->Trigger() == ETRIGGER_UNIT_POSITION) ||
+                   (e->Trigger() == ETRIGGER_UNIT_DESTROYED)) && (key == "tunit") ) { // unit or unit type
           if ( val.size() > 0 && (val[0] == '-' || isdigit(val[0])) ) {
             // seems to be a number -> specific unit or -1
             e->SetTData(0, StrToNum(val));
@@ -654,7 +654,7 @@ int EventHandler::ParseSection( ifstream &in, const string *opt, unsigned long &
             }
             else if ( key == "tile" ) {                                   // new terrain type
               id = StrToNum(val);
-              if ( (id < m.GetTerrainSet().NumTT()) && (id >= 0) ) e->SetData(0, id);
+              if ( (id < m.GetTerrainSet().NumTiles()) && (id >= 0) ) e->SetData(0, id);
               else {
                 rc = 0;
                 cerr << "Error: Invalid tile '" << val << "' near line " << line << endl;
@@ -828,7 +828,7 @@ int MapRawHandler::ParseSection( ifstream &in, const string *opt, unsigned long
         buf[pos++] = '\0';
 
         terrain = atoi( start );
-        if ( terrain >= map.GetTerrainSet()->NumTT() ) {
+        if ( terrain >= map.GetTerrainSet()->NumTiles() ) {
           cerr << "Error in line " << line << ": Invalid hex id '" << terrain << "'" << endl;
           return -1;
         } else map.SetHexType( j, i, terrain );
@@ -1296,10 +1296,10 @@ int MissionParser::check_shops( void ) const {
       }
     }
 
-    if ( b->Crystals() > b->CrystalStore() ) {
+    if ( b->Crystals() > b->MaxCrystals() ) {
       cerr << "Error: Shop at " << b->Position().x << '/' << b->Position().y
                 << " contains " << b->Crystals() << " crystals, but only "
-                << b->CrystalStore() << " fit in" << endl;
+                << b->MaxCrystals() << " fit in" << endl;
       return 1;
     }
 
@@ -1609,7 +1609,7 @@ int MissionParser::check_events( void ) const {
         e->SetTData(2, 0);
         break;
       case ETRIGGER_UNIT_DESTROYED:
-        if ( e->GetTData(0) != -1 ) {
+        if ( e->GetTData(0) >= 0 ) {
           u = GetUnitByID( e->GetTData(0) );
 
           /* the event must also be triggered when the unit is not
@@ -1620,7 +1620,14 @@ int MissionParser::check_events( void ) const {
             cerr << "Error: Event trigger targets non-existing unit with ID " << e->GetTData(0) << endl;
             return 1;
           }
-        } else if ( e->GetTData(1) == -9999 ) e->SetTData(1, e->Player()^1);
+        } else {
+          if ( -e->GetTData(0) - 2 >= unit_set->NumTiles() ) {
+            cerr << "Error: Event trigger targets non-existing unit type " << -e->GetTData(0) - 2
+                 << " (" << (short)e->ID() << ')' << endl;
+            return 1;
+          }
+          if ( e->GetTData(1) == -9999 ) e->SetTData(1, e->Player()^1);
+        }
         e->SetTData(2, 0);
         break;
       case ETRIGGER_HAVE_UNIT:
@@ -1674,7 +1681,7 @@ int MissionParser::check_events( void ) const {
         }
 
         if ( e->GetTData(0) < -1 ) {
-          if ( -e->GetTData(0) - 2 >= unit_set->NumUT() ) {
+          if ( -e->GetTData(0) - 2 >= unit_set->NumTiles() ) {
             cerr << "Error: Event trigger targets non-existing unit type " << -e->GetTData(0) - 2
                  << " (" << (short)e->ID() << ')' << endl;
             return 1;
